'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleFontProperties = handleFontProperties;

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _cssPropertyParser = require('css-property-parser');

var _cssGenericFonts = require('../../constants/cssGenericFonts');

var _cssSystemFonts = require('../../constants/cssSystemFonts');

var _reExistingVendorPrefix = require('../../constants/reExistingVendorPrefix');

var _countUsage = require('../../calculators/countUsage');

var _removeExtraSpaces = require('../../converters/removeExtraSpaces');

var _handleVendorPrefix = require('../handleVendorPrefix');

var _isNumber = require('../../predicates/isNumber');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function countFontSizes(propValue, report) {
	report.fontSizes.total++;
	(0, _countUsage.countUsage)(propValue, report.fontSizes.usage);
}

function countLineHeights(propValue, report) {
	report.lineHeights.total++;
	(0, _countUsage.countUsage)(propValue, report.lineHeights.usage);

	/** Count hard-coded line-heights, i.e `line-height: 16px` */
	if ((0, _isNumber.isNumber)(Number(propValue)) === false) {
		report.lineHeights.hardCoded++;
	}
}

function countFontFamilies(propValue, report) {
	var familiesList = _postcss2.default.list.comma(propValue);

	familiesList.map(function (family) {
		return family.trim().replace(/['"]/g, '').trim();
	}).forEach(function (family) {
		report.fontFamilies.total++;

		if (_reExistingVendorPrefix.reExistingVendorPrefix.test(family)) {
			(0, _handleVendorPrefix.handleVendorPrefix)(family, report);
		}

		if (_cssGenericFonts.cssGenericFonts.includes(family)) {
			(0, _countUsage.countUsage)(family, report.fontFamilies.generic);
		}

		(0, _countUsage.countUsage)(family, report.fontFamilies.usage);
	});

	if (familiesList.length === 1) {
		report.fontFamilies.withoutFallbackFonts++;
	}
}

function handleFontProperties(decl, report) {
	var cleanedValue = (0, _removeExtraSpaces.removeExtraSpaces)(decl.value);

	if (decl.prop === 'font') {
		// http://nicolasgallagher.com/another-css-image-replacement-technique/
		if (/^0\/0 [a-z]/.test(cleanedValue)) {
			report.fontFamilies.imageReplacementHacks++;
		}

		if (_cssSystemFonts.cssSystemFonts.includes(decl.value)) {
			(0, _countUsage.countUsage)(decl.value, report.fontFamilies.system);
			return;
		}

		var fontLonghand = (0, _cssPropertyParser.expandShorthandProperty)(decl.prop, cleanedValue);

		if (typeof fontLonghand['font-size'] === 'string') {
			countFontSizes((0, _removeExtraSpaces.removeExtraSpaces)(fontLonghand['font-size']), report);
		}

		if (typeof fontLonghand['line-height'] === 'string') {
			countLineHeights((0, _removeExtraSpaces.removeExtraSpaces)(fontLonghand['line-height']), report);
		}

		if (typeof fontLonghand['font-family'] === 'string') {
			countFontFamilies((0, _removeExtraSpaces.removeExtraSpaces)(fontLonghand['font-family']), report);
		}
	}

	if (decl.prop === 'font-size') {
		countFontSizes(cleanedValue, report);
	}

	if (decl.prop === 'line-height') {
		countLineHeights(cleanedValue, report);
	}

	if (decl.prop === 'font-family') {
		countFontFamilies(cleanedValue, report);
	}
}