'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleFunctions = handleFunctions;

var _reCssFunction = require('../../constants/reCssFunction');

var _reGradient = require('../../constants/reGradient');

var _reCubicBezier = require('../../constants/reCubicBezier');

var _reUrlFunctionWithArg = require('../../constants/reUrlFunctionWithArg');

var _reImageDataUri = require('../../constants/reImageDataUri');

var _countUsage = require('../../calculators/countUsage');

var _removeExtraSpaces = require('../../converters/removeExtraSpaces');

var _isValidCubicBezierArgs = require('../../predicates/isValidCubicBezierArgs');

function countCubicBeziers(decl, report, options) {
	if (options.collectTransitionsAndAnimationsData) {
		var reportSection = decl.prop.includes('animation') ? report.animations : report.transitions;

		decl.value.match(_reCubicBezier.reCubicBezier).forEach(function (func) {
			var processedFunc = (0, _removeExtraSpaces.removeExtraSpaces)(func);
			(0, _countUsage.countUsage)(processedFunc, reportSection.timingFunctions);

			/**
    * Count invalid cubic beziers
    * https://developer.mozilla.org/en-US/docs/Web/CSS/single-transition-timing-function#Examples
    */
			if ((0, _isValidCubicBezierArgs.isValidCubicBezierArgs)(processedFunc) === false) {
				(0, _countUsage.countUsage)(processedFunc, reportSection.invalidTimingFunctions);
			}
		});
	}
}

function handleFunctions(decl, report, options) {
	if (_reCssFunction.reCssFunction.test(decl.value)) {
		decl.value.match(_reCssFunction.reCssFunction).map(function (func) {
			return func.slice(0, func.indexOf('('));
		}).forEach(function (func) {
			report.functions.total++;
			(0, _countUsage.countUsage)(func, report.functions.usage);
		});

		if (options.collectDataUrisData && _reUrlFunctionWithArg.reUrlFunctionWithArg.test(decl.value)) {
			decl.value.match(_reUrlFunctionWithArg.reUrlFunctionWithArg).map(function (func) {
				return (0, _removeExtraSpaces.removeExtraSpaces)(func).replace(/^url\(['"]?/g, '').replace(/['"]?\)/g, '');
			}).forEach(function (urlArg) {
				if (urlArg.match(_reImageDataUri.reImageDataUri) !== null) {
					report.dataUris.total++;
					report.dataUris.totalByteLength += Buffer.byteLength(urlArg, 'utf8');
					(0, _countUsage.countUsage)(urlArg, report.dataUris.usage);
				}
			});
		}

		if (options.collectGradientsData && _reGradient.reGradient.test(decl.value)) {
			decl.value.match(_reGradient.reGradient).map(function (gradient) {
				return (0, _removeExtraSpaces.removeExtraSpaces)(gradient);
			}).forEach(function (gradient) {
				report.gradients.total++;
				(0, _countUsage.countUsage)(gradient, report.gradients.usage);
			});
		}

		if (_reCubicBezier.reCubicBezier.test(decl.value)) {
			countCubicBeziers(decl, report, options);
		}
	}
}