'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleSelector = handleSelector;

var _postcssSelectorParser = require('postcss-selector-parser');

var _postcssSelectorParser2 = _interopRequireDefault(_postcssSelectorParser);

var _cssPseudoClasses = require('../../constants/cssPseudoClasses');

var _cssPseudoElements = require('../../constants/cssPseudoElements');

var _rePrefixedString = require('../../constants/rePrefixedString');

var _calculateSpecificity = require('../../calculators/calculateSpecificity');

var _compareSpecificities = require('../../calculators/compareSpecificities');

var _countUsage = require('../../calculators/countUsage');

var _mergeSpecificities = require('../../converters/mergeSpecificities');

var _handleVendorPrefix = require('../handleVendorPrefix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var processedSelectors = null;
function selectorsProcessor(selectors) {
	processedSelectors = selectors;
}

var selectorParser = (0, _postcssSelectorParser2.default)(selectorsProcessor);

var combinatorsMap = {
	'+': 'adjacentSibling',
	'>': 'child',
	' ': 'descendant',
	'~': 'generalSibling'
};

function isAllowedSelectorType(type) {
	return ['attribute', 'class', 'id', 'pseudo', 'tag', 'universal'].includes(type);
}

var reLeadingColons = /^:{1,2}/;

function countSelectors(selectors, report, options) {
	var tmpSpecificity = [0, 0, 0];
	var baseSelectorsWithoutCombinators = 0;

	selectors.walk(function (selector) {
		if (selector.type === 'combinator') {
			report.selectors.combinators.total++;
			report.selectors.combinators[combinatorsMap[selector.value]]++;
		}

		if (isAllowedSelectorType(selector.type)) {
			baseSelectorsWithoutCombinators++;

			if (selector.type === 'pseudo') {
				var pseudoName = selector.value.replace(reLeadingColons, '');

				if (_rePrefixedString.rePrefixedString.test(pseudoName)) {
					(0, _handleVendorPrefix.handleVendorPrefix)(pseudoName, report);
				}

				if (_cssPseudoElements.cssPseudoElements.some(function (name) {
					return pseudoName === name;
				})) {
					(0, _countUsage.countUsage)('pseudoElement', report.selectors.baseUsage);
					(0, _countUsage.countUsage)(selector.value.replace(reLeadingColons, '::'), report.selectors.pseudoElementsUsage);
				} else if (_cssPseudoClasses.cssPseudoClasses.some(function (name) {
					return pseudoName === name;
				})) {
					(0, _countUsage.countUsage)('pseudoClass', report.selectors.baseUsage);
					(0, _countUsage.countUsage)(selector.value.replace(reLeadingColons, ':'), report.selectors.pseudoClassesUsage);
				}
			} else {
				(0, _countUsage.countUsage)(selector.type, report.selectors.baseUsage);
			}

			var specificity = (0, _calculateSpecificity.calculateSpecificity)(selector);

			tmpSpecificity = (0, _mergeSpecificities.mergeSpecificities)(tmpSpecificity, specificity);
			report.selectors.totalSpecificity = (0, _mergeSpecificities.mergeSpecificities)(report.selectors.totalSpecificity, specificity);

			if (options.collectSpecificityGraphData) {
				report.selectors.specificityGraphData.push(specificity);
			}
		}
	});

	if ((0, _compareSpecificities.compareSpecificities)(report.selectors.highestSpecificity, tmpSpecificity) === 1) {
		report.selectors.highestSpecificity = tmpSpecificity;
		report.selectors.highestSpecificitySelector = String(selectors);
	}

	if (baseSelectorsWithoutCombinators > options.selectorComplexityThreshold) {
		report.selectors.complex++;
	}
}

function handleSelector(selector, report, options) {
	report.selectors.total++;

	if (options.collectSelectorsUsage) {
		(0, _countUsage.countUsage)(selector, report.selectors.usage);
	}

	var selectorByteLength = Buffer.byteLength(selector, 'utf8');
	report.selectors.totalByteLength += selectorByteLength;

	if (report.selectors.longestByteLength < selectorByteLength) {
		report.selectors.longestByteLength = selectorByteLength;
		report.selectors.longestByteLengthSelector = selector;
	}

	selectorParser.processSync(selector);

	if (processedSelectors !== null) {
		countSelectors(processedSelectors, report, options);
	}
}